@@ -317,6 +317,7 @@ class ExecutionMagics(Magics):
             the magic line is always left unmodified.
 
         """
+        # TODO: port to magic_arguments as currently this is duplicated in IPCompleter._extract_code
         opts, arg_str = self.parse_options(parameter_s, 'D:l:rs:T:q',
                                            list_all=True, posix=False)
         if cell is not None:
@@ -441,11 +442,10 @@ class ExecutionMagics(Magics):
         Set break point at LINE in FILE.
         """
     )
-    @magic_arguments.argument('statement', nargs='*',
-        help="""
-        Code to run in debugger.
-        You can omit this in cell magic mode.
-        """
+    @magic_arguments.kwds(
+        epilog="""
+      Any remaining arguments will be treated as code to run in the debugger.
+    """
     )
     @no_var_expand
     @line_cell_magic
@@ -454,12 +454,12 @@ class ExecutionMagics(Magics):
         """Activate the interactive debugger.
 
         This magic command support two ways of activating debugger.
-        One is to activate debugger before executing code.  This way, you
+        One is to activate debugger before executing code. This way, you
         can set a break point, to step through the code from the point.
         You can use this mode by giving statements to execute and optionally
         a breakpoint.
 
-        The other one is to activate debugger in post-mortem mode.  You can
+        The other one is to activate debugger in post-mortem mode. You can
         activate this mode simply running %debug without any argument.
         If an exception has just occurred, this lets you inspect its stack
         frames interactively.  Note that this will always work only on the last
@@ -475,9 +475,9 @@ class ExecutionMagics(Magics):
             the magic line is always left unmodified.
 
         """
-        args = magic_arguments.parse_argstring(self.debug, line)
+        args, extra = magic_arguments.parse_argstring(self.debug, line, partial=True)
 
-        if not (args.breakpoint or args.statement or cell):
+        if not (args.breakpoint or extra or cell):
             self._debug_post_mortem()
         elif not (args.breakpoint or cell):
             # If there is no breakpoints, the line is just code to execute
@@ -486,7 +486,7 @@ class ExecutionMagics(Magics):
             # Here we try to reconstruct the code from the output of
             # parse_argstring. This might not work if the code has spaces
             # For example this fails for `print("a b")`
-            code = "\n".join(args.statement)
+            code = " ".join(extra)
             if cell:
                 code += "\n" + cell
             self._debug_exec(code, args.breakpoint, local_ns)
@@ -1138,6 +1138,7 @@ class ExecutionMagics(Magics):
         does not matter as long as results from timeit.py are not mixed with
         those from ``%timeit``."""
 
+        # TODO: port to magic_arguments as currently this is duplicated in IPCompleter._extract_code
         opts, stmt = self.parse_options(
             line, "n:r:tcp:qov:", posix=False, strict=False, preserve_non_opts=True
         )
@@ -1266,6 +1267,11 @@ class ExecutionMagics(Magics):
         dest="no_raise_error",
         help="If given, don't re-raise exceptions",
     )
+    @magic_arguments.kwds(
+        epilog="""
+      Any remaining arguments will be treated as code to run.
+    """
+    )
     @skip_doctest
     @needs_local_scope
     @line_cell_magic
@@ -1337,34 +1343,10 @@ class ExecutionMagics(Magics):
                 Wall time: 0.00 s
                 Compiler : 0.78 s
         """
-        line_present = False
-        # Try to parse --no-raise-error if present, else ignore unrecognized args
-        try:
-            args = magic_arguments.parse_argstring(self.time, line)
-        except UsageError as e:
-            # Only ignore UsageError if caused by unrecognized arguments
-            # We'll manually check for --no-raise-error and remove it from line
-            line_present = True
+        args, extra = magic_arguments.parse_argstring(self.time, line, partial=True)
+        line = " ".join(extra)
 
-            # Check if --no-raise-error is present
-            no_raise_error = "--no-raise-error" in line
-
-            if no_raise_error:
-                # Remove --no-raise-error while preserving the rest of the line structure
-                line = re.sub(r"\s*--no-raise-error\s*", " ", line).strip()
-                # Clean up any double spaces
-                line = re.sub(r"\s+", " ", line)
-
-            class Args:
-                def __init__(self, no_raise_error):
-                    self.no_raise_error = no_raise_error
-
-            args = Args(no_raise_error)
-        else:
-            if not hasattr(args, "no_raise_error"):
-                args.no_raise_error = False
-
-        if line_present and cell:
+        if line and cell:
             raise UsageError("Can't use statement directly after '%%time'!")
 
         if cell:
