@@ -247,6 +247,8 @@ class InteractiveShellApp(Configurable):
     def init_path(self):
         """Add current working directory, '', to sys.path
 
+        Unless disabled by ignore_cwd config or sys.flags.safe_path.
+
         Unlike Python's default, we insert before the first `site-packages`
         or `dist-packages` directory,
         so that it is after the standard library.
@@ -255,8 +257,10 @@ class InteractiveShellApp(Configurable):
             Try to insert after the standard library, instead of first.
         .. versionchanged:: 8.0
             Allow optionally not including the current directory in sys.path
+        .. versionchanged:: 9.7
+            Respect sys.flags.safe_path (PYTHONSAFEPATH and -P flag)
         """
-        if '' in sys.path or self.ignore_cwd:
+        if "" in sys.path or self.ignore_cwd or sys.flags.safe_path:
             return
         for idx, path in enumerate(sys.path):
             parent, last_part = os.path.split(path)
