@@ -5,12 +5,12 @@ from IPython import embed
 a = 10
 b = 20
 
-embed(header='First time', banner1='')
+embed(header="First time", banner1="")
 
 c = 30
 d = 40
 
 try:
-    raise Exception('adsfasdf')
+    raise Exception("adsfasdf")
 except:
-    embed(header='The second time')
+    embed(header="The second time")
