@@ -170,6 +170,10 @@ def doctest_reset_del():
 # For some tests, it will be handy to organize them in a class with a common
 # setup that makes a temp file
 
+import sysconfig
+
+is_freethreaded = bool(sysconfig.get_config_var("Py_GIL_DISABLED"))
+
 
 class TestMagicRunPass(tt.TempFileMixin):
 
@@ -294,6 +298,7 @@ class TestMagicRunSimple(tt.TempFileMixin):
             _ip.run_line_magic("run", empty.fname)
             assert _ip.user_ns["afunc"]() == 1
 
+    @pytest.mark.xfail(is_freethreaded, reason="C-third leaks on free-threaded python")
     def test_tclass(self):
         mydir = os.path.dirname(__file__)
         tc = os.path.join(mydir, "tclass")
